# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the hashtaglist section
'''Get values of the hashtaglist section'''

# standard library imports
import os.path
import sys

def parsehashtaglist(clioption, config):
    '''Parse configuration values and get values of the hashtaglist section'''
    hashtaglist = ''
    section = 'hashtaglist'
    if not clioption:
        ####################################
        # several_words_hashtags_list option
        ####################################
        confoption = 'several_words_hashtags_list'
        if config.has_section(section):
            if config.has_option(section, confoption):
                hashtaglist = config.get(section, confoption)
                hashtaglist = os.path.expanduser(hashtaglist)
                if not os.path.exists(hashtaglist) or not os.path.isfile(hashtaglist):
                    sys.exit('The path to the several_words_hashtags_list parameter is not valid: {hashtaglist}'.format(hashtaglist=hashtaglist))
    return hashtaglist



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the media section
'''Get values of the media section'''

# standard library imports
import os.path
import sys

def parsemedia(config):
    '''Parse configuration values and get values of the media section'''
    mediaconf = {}
    section = 'media'
    ####################################
    # media option
    ####################################
    confoption = 'custom'
    if config.has_section(section):
        if config.has_option(section, confoption):
            media = config.get(section, confoption)
            media = os.path.expanduser(media)
            if not os.path.exists(media) or not os.path.isfile(media):
                sys.exit('The path to the custom parameter is not valid: {media}'.format(media=media))
            else:
                mediaconf[confoption] = media
    return mediaconf



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get value of the uri option of rss section
'''Get value of the uri option of the rss section'''

# standard library imports
import feedparser
import sys
import re

def parseuri(config, clioption, feeds):
    '''Parse configuration value of the uri option of the rss section'''
    rssuri = ''
    feedname =''
    nopatternurinoglobalpattern = False
    section = 'rss'
    if config.has_section(section):
        ############
        # uri option
        ############
        if not feeds and not clioption:
            confoption = 'uri'
            if config.has_option(section, confoption):
                urifeed = config.get('rss', 'uri')
                feedname = None
                if '<' in urifeed:
                    matches = re.match('(.*) <(.*)>', urifeed)
                    if not matches:
                        sys.exit('This uri to parse is not formatted correctly: {urifeed}'.format(urifeed))
                    feedname, finaluri = matches.groups()
                    rssuri = finaluri
                else:
                    rssuri = config.get('rss', 'uri')
            else:
                sys.exit('{confoption} parameter in the [{section}] section of the configuration file is mandatory. Exiting.'.format(section=section, confoption=confoption))
        else:
            rssuri = clioption
        # get the rss feed for rss parameter of [rss] section
        feed = feedparser.parse(rssuri)
        if not feed:
            sys.exit('Unable to parse the feed at the following url: {rss}'.format(rss=rss))
        #########################################
        # no_uri_pattern_no_global_pattern option
        #########################################
        currentoption = 'no_uri_pattern_no_global_pattern'
        # default value
        if config.has_option(section, currentoption):
            nopatternurinoglobalpattern = config.getboolean(section, currentoption)
        return rssuri, feed, feedname, nopatternurinoglobalpattern



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the configuration file
'''Get values of the configuration file'''

# standard library imports
from configparser import SafeConfigParser
import logging
import os
import os.path
import sys
import re

# 3rd party library imports
import feedparser

# feed2toot library imports
from feed2toot.confparsers.cache import parsecache
from feed2toot.confparsers.hashtaglist import parsehashtaglist
from feed2toot.confparsers.feedparser import parsefeedparser
from feed2toot.confparsers.lock import parselock
from feed2toot.confparsers.media import parsemedia
from feed2toot.confparsers.plugins import parseplugins
from feed2toot.confparsers.rss.pattern import parsepattern
from feed2toot.confparsers.rss.toot import parsetoot
from feed2toot.confparsers.rss.uri import parseuri
from feed2toot.confparsers.rss.urilist import parseurilist
from feed2toot.confparsers.rss.addtags import parseaddtags

class ConfParse:
    '''ConfParse class'''
    def __init__(self, clioptions):
        '''Constructor of the ConfParse class'''
        self.clioptions = clioptions
        self.tweetformat = ''
        self.stringsep = ','
        self.confs = []
        self.main()

    def main(self):
        '''Main of the ConfParse class'''
        for pathtoconfig in self.clioptions.configs:
            options = {}
            # read the configuration file
            config = SafeConfigParser()
            if not config.read(os.path.expanduser(pathtoconfig)):
                sys.exit('Could not read config file')
            ####################
            # feedparser section
            ####################
            accept_bozo_exceptions = parsefeedparser(config)
            ###########################
            # the rss section
            ###########################
            self.tweetformat = parsetoot(config)
            #################################################
            # pattern and patter_case_sensitive format option
            #################################################
            options['patterns'], options['patternscasesensitive'] = parsepattern(config)
            #################################################
            # lock file options
            #################################################
            options['lockfile'], options['locktimeout'] = parselock(self.clioptions.lockfile, self.clioptions.locktimeout, config)
            ###############################
            # addtags option, default: True
            ###############################
            options['addtags'] = parseaddtags(config)
            #################
            # uri_list option
            #################
            feeds = []
            feeds = parseurilist(config, accept_bozo_exceptions)
            ############
            # uri option
            ############
            options['rss_uri'], feed, feedname, options['nopatternurinoglobalpattern']  = parseuri(config, self.clioptions.rss_uri, feeds)
            ###########################
            # the cache section
            ###########################
            options['cachefile'], options['cache_limit'] = parsecache(self.clioptions.cachefile, config)
            ###########################
            # the hashtag section
            ###########################
            options['hashtaglist'] = parsehashtaglist(self.clioptions.hashtaglist, config)
            ###########################
            # the media section
            ###########################
            options['media'] = parsemedia(config)
            ###########################
            # the plugins section
            ###########################
            plugins = parseplugins(config)
            ########################################
            # return the final configurations values
            ########################################
            if feeds:
                self.confs.append((options, config, self.tweetformat, feeds, plugins))
            else:
                self.confs.append((options, config, self.tweetformat, [{'feed': feed, 'patterns': [], 'rssobject': '', 'feedname': feedname}], plugins))

    @property
    def confvalues(self):
        '''Return the values of the different configuration files'''
        return self.confs



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Filter an entry of the RSS feeds
'''Filter an entry of the RSS feeds'''

# standard library imports
import sys

class FilterEntry:
    '''FilterEntry class'''
    def __init__(self, elements, entry, options, byrsspatterns, rssobject, feedname):
        '''Constructor of the FilterEntry class'''
        self.matching = {}
        self.entry = entry
        self.elements = elements
        self.options = options
        self.byrsspatterns = byrsspatterns
        self.rssobject = rssobject
        self.feedname = feedname
        self.main()

    def main(self):
        '''Main of the FilterEntry class'''
        authorized_elements = ['feedname', ]
        authorized_elements.extend(self.entry.keys())
        for i in self.elements:
            if i not in authorized_elements:
                sys.exit('The element {} is not available in the RSS feed. The available ones are: {}'.format(i, [j for j in self.entry]))
            # for the case if no pattern at all is defined
            if i == 'feedname':
                self.matching[i] = self.feedname
            elif not self.options['patterns'] and not self.byrsspatterns and not self.rssobject:
                self.matching[i] = self.entry[i]
            # global filter only
            elif self.options['patterns'] and not self.byrsspatterns and not self.rssobject:
                if not self.options['nopatternurinoglobalpattern']:
                    self.applyglobalfilter(i)
                else:
                    self.matching[i] = self.entry[i]
            # global filter and then by rss filter
            elif self.options['patterns'] and self.byrsspatterns and self.rssobject:
                # patterns by rss
                self.applyglobalfilter(i)
                self.applyspecificfilter(i)
            elif not self.options['patterns'] and self.byrsspatterns and self.rssobject:
                self.applyspecificfilter(i)
            else:
                self.matching[i] = self.entry[i]

    def applyglobalfilter(self, i):
        '''Apply the global filter'''
        for patternlist in self.options['patterns']:
            if not self.options['patternscasesensitive']['{}_case_sensitive'.format(patternlist)]:
                # not case sensitive, so we compare the lower case
                for pattern in self.options['patterns'][patternlist]:
                    finalpattern = pattern.lower()
                    finaltitle = self.entry[patternlist.split('_')[0]].lower()
                    if finalpattern in finaltitle:
                        self.matching[i] = self.entry[i]
            else:
                # case sensitive, so we use the user-defined pattern
                for pattern in self.options['patterns'][patternlist]:
                    if pattern in self.entry['title']:
                        self.matching[i] = self.entry[i]

    def applyspecificfilter(self, i):
        '''Apply specific filters for by-rss pattern matching'''
        for byrsspattern in self.byrsspatterns:
            byrssfinalpattern = byrsspattern.lower()
            if byrssfinalpattern in self.entry[self.rssobject].lower():
                self.matching[i] = self.entry[i]

    @property
    def finalentry(self):
        '''Return the processed entry'''
        return self.matching



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Checks an RSS feed and posts new entries to Mastodon."""

# 3rd party libraries imports
from mastodon import Mastodon

class TootPost:
    '''TootPost class'''

    def __init__(self, config, options, toot):
        '''Constructore of the TootPost class'''
        self.config = config
        self.options = options
        self.store = True
        self.toot = toot
        self.main()

    def main(self):
        '''Main of the TweetPost class'''
        mastodon = Mastodon(
            client_id=self.config.get('mastodon', 'client_credentials'),
            access_token=self.config.get('mastodon', 'user_credentials'),
            api_base_url=self.config.get('mastodon', 'instance_url')
        )
        toot_visibility = self.config.get('mastodon', 'toot_visibility', fallback='public')
        if 'custom' in self.options['media']:
            mediaid = mastodon.media_post(self.config['media']['custom'])
            mastodon.status_post(self.toot, media_ids=[mediaid], visibility=toot_visibility)
        else:
            mastodon.status_post(self.toot, visibility=toot_visibility)

    def storeit(self):
        '''Indicate if the tweet should be stored or not'''
        return self.store



#!/usr/bin/env python3
# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Remove duplicates from the final string before sending the tweet
'''Remove duplicates from the final string before sending the tweet'''

class RemoveDuplicates:
    '''Remove duplicates from the final string before sending the tweet'''
    def __init__(self, tweet):
        '''Constructor of RemoveDuplicates class'''
        self.tweet = tweet
        self.main()

    def main(self):
        '''Main of the RemoveDuplicates class'''
        # identify duplicate links
        links = []
        for element in self.tweet.split():
            if element != ' ' and (element.startswith('http://') or element.startswith('https://')):
                newlink = True
                # if we already found this link, increment the counter
                for i, _ in enumerate(links):
                    if links[i]['link'] == element:
                        newlink = False
                        links[i]['count'] += 1
                if newlink:
                    links.append({'link': element, 'count': 1})
        # remove duplicates
        validatedlinks = []
        for i in range(len(links)):
            if links[i]['count'] >= 2:
                validatedlinks.append(links[i])
        wildcard = 'FEED2TOOTWILDCARD'
        for element in validatedlinks:
            for i in range(element['count']):
                # needed for not inversing the order of links if it is a duplicate
                # and the second link is not one
                if i == 0:
                    self.tweet = self.tweet.replace(element['link'], wildcard, 1)
                else:
                    self.tweet = self.tweet.replace(element['link'], '', 1)
            # finally
            self.tweet = self.tweet.replace(wildcard, element['link'], 1)
        # remove all 2xspaces
        self.tweet = self.tweet.replace('  ', ' ')

    @property
    def finaltweet(self):
        '''return the final tweet after duplicates were removed'''
        return self.tweet



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get value of the toot/tweet option of rss section
'''Get value of the toot/tweet option of the rss section'''

# standard library imports
import sys
import logging

def parsetoot(config):
    '''Parse configuration value of the toot/tweet optionof the rss section'''
    section = 'rss'
    if config.has_section(section):
        ############################
        # tweet option
        ############################
        oldconfoption = 'tweet'
        confoption = 'toot'
        # manage 'tweet' for compatibility reason with first versions
        if config.has_option(section, oldconfoption):
            logging.warn("Your configuration file uses a 'tweet' parameter instead of 'toot'. 'tweet' is deprecated and will be removed in Feed2toot 0.7")
            tootformat = config.get(section, oldconfoption)
        elif config.has_option(section, confoption):
            tootformat = config.get(section, confoption)
        else:
            sys.exit('You should define a format for your tweet with the parameter "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
    return tootformat



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the plugins section
'''Get values of the plugins section'''

# standard library imports
import sys

def parseplugins(config):
    '''Parse configuration values and get values of the plugins section'''
    plugins = {}
    section = 'influxdb'
    if config.has_section(section):
        ##########################################
        # host, port, user, pass, database options
        ##########################################
        plugins[section] = {}
        for currentoption in ['host', 'port', 'user', 'pass', 'database', 'measurement']:
            if config.has_option(section, currentoption):
                plugins[section][currentoption] = config.get(section, currentoption)
        if 'host' not in plugins[section]:
            plugins[section]['host'] = '127.0.0.1'
        if 'port' not in plugins[section]:
            plugins[section]['port'] = 8086
        if 'measurement' not in plugins[section]:
            plugins[section]['measurement'] = 'toots'
        for field in ['user', 'pass', 'database']:
            if field not in plugins[section]:
                sys.exit('Parsing error for {field} in the [{section}] section: {field} is not defined'.format(field=field, section=section))
    return plugins



#!/usr/bin/env python3
# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Checks an RSS feed and posts new entries to Mastodon."""

# standard libraires imports
import codecs
import importlib
import logging
import logging.handlers
import sys
import re

# app libraries imports
from feed2toot.addtags import AddTags
from feed2toot.cliparse import CliParse
from feed2toot.confparse import ConfParse
from feed2toot.filterentry import FilterEntry
from feed2toot.removeduplicates import RemoveDuplicates
from feed2toot.tootpost import TootPost
from feed2toot.feedcache import FeedCache
from feed2toot.lock import LockFile
from bs4 import BeautifulSoup

class Main:
    '''Main class of Feed2toot'''

    def __init__(self):
        self.main()

    def setup_logging(self, options):
        if options.syslog:
            sl = logging.handlers.SysLogHandler(address='/dev/log')
            sl.setFormatter(logging.Formatter('feed2toot[%(process)d]: %(message)s'))
            # convert syslog argument to a numeric value
            loglevel = getattr(logging, options.syslog.upper(), None)
            if not isinstance(loglevel, int):
                raise ValueError('Invalid log level: %s' % loglevel)
            sl.setLevel(loglevel)
            logging.getLogger('').addHandler(sl)
            logging.debug('configured syslog level %s' % loglevel)
        logging.getLogger('').setLevel(logging.DEBUG)
        sh = logging.StreamHandler()
        sh.setLevel(options.log_level.upper())
        logging.getLogger('').addHandler(sh)
        logging.debug('configured stdout level %s' % sh.level)

    def main(self):
        """The main function."""
        clip = CliParse()
        clioptions = clip.options
        self.setup_logging(clioptions)
        # iterating over the different configuration files
        cfgp = ConfParse(clioptions)
        confs = cfgp.confvalues
        for conf in confs:
            options = conf[0]
            config = conf[1]
            tweetformat = conf[2]
            feeds = conf[3]
            plugins = conf[4]
            # check the logfile and logtimeout
            lockfile = LockFile(options['lockfile'], options['locktimeout'])
            # create link to the persistent list
            cache = FeedCache(options)
            if 'hashtaglist' in options and options['hashtaglist']:
                severalwordshashtags = codecs.open(options['hashtaglist'],
                                                   encoding='utf-8').readlines()
                severalwordshashtags = [i.rstrip('\n') for i in severalwordshashtags]
            else:
                severalwordshashtags = []
            # reverse feed entries because most recent one should be sent as the last one in Mastodon
            for feed in feeds:
                # store the patterns by rss
                if 'patterns' in feed:
                    patterns = feed['patterns']
                entries = feed['feed']['entries'][0:clioptions.limit]
                entries.reverse()
                # --rss-sections option: print rss sections and exit
                if clioptions.rsssections:
                    if entries:
                        print('The following sections are available in this RSS feed: {}'.format([j for j in entries[0]]))
                        sys.exit(0)
                    else:
                        sys.exit('Could not parse the section of the rss feed')
                totweet = []
                # cache the ids of last rss feeds
                if not clioptions.all:
                    for i in entries:
                        if 'id' in i:
                            if i['id'] not in cache.getdeque():
                                totweet.append(i)
                        elif 'guid' in i:
                            if i['guid'] not in cache.getdeque():
                                totweet.append(i)
                        else:
                            # if id or guid not in the entry, use link
                            if i['link'] not in cache.getdeque():
                                totweet.append(i)
                else:
                    totweet = entries

                for entry in totweet:
                    if 'id' in entry:
                        logging.debug('found feed entry {entryid}'.format(entryid=entry['id']))
                        rss = {
                            'id': entry['id'],
                        }
                    elif 'guid' in entry:
                        logging.debug('found feed entry {entryid}'.format(entryid=entry['guid']))
                        rss = {
                            'id': entry['guid'],
                        }
                    else:
                        logging.debug('found feed entry {entryid}'.format(entryid=entry['link']))
                        rss = {
                            'id': entry['link'],
                        }

                    severalwordsinhashtag = False
                    # lets see if the rss feed has hashtag
                    if 'tags' in entry and options['addtags']:
                        hastags = True
                    else:
                        hastags = False
                    if hastags:
                        rss['hashtags'] = []
                        for i, _ in enumerate(entry['tags']):
                            if 'hashtaglist' in options:
                                prehashtags = entry['tags'][i]['term']
                                tmphashtags = entry['tags'][i]['term']
                                for element in severalwordshashtags:
                                    if element in prehashtags:
                                        severalwordsinhashtag = True
                                        tmphashtags = prehashtags.replace(element,
                                                                          ''.join(element.split()))
                            # replace characters stopping a word from being a hashtag
                            if severalwordsinhashtag:
                                # remove ' from hashtag
                                tmphashtags = tmphashtags.replace("'", "")
                                # remove - from hashtag
                                tmphashtags = tmphashtags.replace("-", "")
                                # remove . from hashtag
                                tmphashtags = tmphashtags.replace(".", "")
                                # remove space from hashtag
                                finalhashtags = tmphashtags.replace(" ", "")
                                rss['hashtags'].append('#{}'.format(finalhashtags))
                            else:
                                nospace = ''.join(entry['tags'][i]['term'])
                                # remove space from hashtag
                                nospace = nospace.replace(" ", "")
                                rss['hashtags'].append('#{}'.format(nospace))
                    # parse tweetfomat to elements
                    elements = re.findall(r"\{(.*?)\}",tweetformat)
                    # strip : from elements to allow string formating, eg. {title:.20}
                    for i,s in enumerate(elements):
                         if s.find(':'):
                             elements[i] = s.split(':')[0]
                    fe = FilterEntry(elements, entry, options, feed['patterns'], feed['rssobject'], feed['feedname'])
                    entrytosend = fe.finalentry
                    if entrytosend:
                        tweetwithnotag = tweetformat.format(**entrytosend)
                        # remove duplicates from the final tweet
                        dedup = RemoveDuplicates(tweetwithnotag)
                        # only append hashtags if they exist
                        # remove last tags if tweet too long
                        if 'hashtags' in rss:
                            addtag = AddTags(dedup.finaltweet, rss['hashtags'])
                            finaltweet = addtag.finaltweet
                        else:
                            finaltweet = dedup.finaltweet

                        # strip html tags
                        finaltweet = BeautifulSoup(finaltweet, 'html.parser').get_text()

                    if clioptions.dryrun:
                        if entrytosend:
                            logging.warning('Would toot with visibility "{visibility}": {toot}'.format(
                                toot=finaltweet,
                                visibility=config.get(
                                    'mastodon', 'toot_visibility',
                                    fallback='public')))
                        else:
                            logging.debug('This rss entry did not meet pattern criteria. Should have not been sent')
                    else:
                        storeit = True
                        if entrytosend and not clioptions.populate:
                            logging.debug('Tooting with visibility "{visibility}": {toot}'.format(
                                toot=finaltweet,
                                visibility=config.get(
                                    'mastodon', 'toot_visibility',
                                    fallback='public')))
                            twp = TootPost(config, options, finaltweet)
                            storeit = twp.storeit()
                        else:
                            logging.debug('populating RSS entry {}'.format(rss['id']))
                        # in both cas we store the id of the sent tweet
                        if storeit:
                            cache.append(rss['id'])
                        # plugins
                        if plugins and entrytosend:
                            for plugin in plugins:
                                capitalizedplugin = plugin.title()
                                pluginclassname = '{plugin}Plugin'.format(plugin=capitalizedplugin)
                                pluginmodulename = 'feed2toot.plugins.{pluginmodule}'.format(pluginmodule=pluginclassname.lower())
                                try:
                                    pluginmodule = importlib.import_module(pluginmodulename)
                                    pluginclass = getattr(pluginmodule, pluginclassname)
                                    pluginclass(plugins[plugin], finaltweet)
                                except ImportError as err:
                                    print(err)
            # do not forget to close cache (shelf object)
            cache.close()
            # release the lock file
            lockfile.release()



# -*- coding: utf-8 -*-
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Push values to a influxdb database
'''Push values to a influxdb database'''

# 3rd party libraries imports
from influxdb import InfluxDBClient

class InfluxdbPlugin(object):
    '''InfluxdbPlugin class'''
    def __init__(self, plugininfo, data):
        '''Constructor of the InfluxdbPlugin class'''
        self.plugininfo = plugininfo
        self.data = data
        self.datatoinfluxdb = []
        self.client = InfluxDBClient(self.plugininfo['host'],
            self.plugininfo['port'],
            self.plugininfo['user'],
            self.plugininfo['pass'],
            self.plugininfo['database'])
        self.main()

    def main(self):
        '''Main of the PiwikModule class'''
        self.datatoinfluxdb.append({
            'measurement': self.plugininfo['measurement'],
            'fields': {'value': self.data}
        })
        self.client.write_points(self.datatoinfluxdb)



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get value of the patterne option of rss section
'''Get value of the pattern option of the rss section'''

# standard library imports
import logging
import sys

def parsepattern(config):
    '''Parse configuration value of the pattern option of the rss section'''
    patterns = {}
    patternscasesensitive = {}
    stringsep = ','
    section = 'rss'
    if config.has_section(section):
        #######################
        # pattern format option
        #######################
        for pattern in ['summary_detail', 'published_parsed', 'guidislink', 'authors', 'links', 'title_detail', 'author', 'author_detail', 'comments', 'published', 'summary', 'tags', 'title', 'link', 'id']:
            currentoption = '{}_pattern'.format(pattern)
            if config.has_option(section, currentoption):
                tmppattern = config.get(section, currentoption)
                if stringsep in tmppattern:
                    patterns[currentoption] = [i for i in tmppattern.split(stringsep) if i]
                else:
                    patterns[currentoption] = [tmppattern]

            ###############################
            # pattern_case_sensitive option
            ###############################
            currentoption = '{}_pattern_case_sensitive'.format(pattern)
            if config.has_option(section, currentoption):
                try:
                    patternscasesensitive[currentoption] = config.getboolean(section, currentoption)
                except ValueError as err:
                    logging.warn(err)
                    patternscasesensitive[currentoption] = True
    return patterns, patternscasesensitive



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# CLI parsing
'''CLI parsing'''

# standard library imports
from argparse import ArgumentParser
import glob
import logging
import os
import os.path
import sys

__version__ = '0.11'

class CliParse:
    '''CliParse class'''
    def __init__(self):
        '''Constructor for the CliParse class'''
        self.main()

    def main(self):
        '''main of CliParse class'''
        feed2tootepilog = 'For more information: https://feed2toot.readhthedocs.org'
        feed2tootdescription = 'Take rss feed and send it to Mastodon'
        parser = ArgumentParser(prog='feed2toot',
                                description=feed2tootdescription,
                                epilog=feed2tootepilog)
        parser.add_argument('--version', action='version', version=__version__)
        parser.add_argument('-c', '--config',
                            default=[os.path.join(os.getenv('XDG_CONFIG_HOME', '~/.config'),
                                                  'feed2toot.ini')],
                            nargs='+',
                            dest="config",
                            help='Location of config file (default: %(default)s)',
                            metavar='FILE')
        parser.add_argument('-a', '--all', action='store_true', default=False,
                            dest='all',
                            help='tweet all RSS items, regardless of cache')
        parser.add_argument('-l', '--limit', dest='limit', default=10, type=int,
                            help='tweet only LIMIT items (default: %(default)s)')
        parser.add_argument('-t', '--lock-timeout', dest='locktimeout', default=3600, type=int,
                            help='lock timeout in seconds after which feed2toot can removes the lock itself')
        parser.add_argument('--cachefile', dest='cachefile',
                            help='location of the cache file (default: %(default)s)')
        parser.add_argument('--lockfile', dest='lockfile',
                            default=os.path.join(os.getenv('XDG_CONFIG_HOME', '~/.config'),
                                                  'feed2toot.lock'),
                            help='location of the lock file (default: %(default)s)')
        parser.add_argument('-n', '--dry-run', dest='dryrun',
                            action='store_true', default=False,
                            help='Do not actually post tweets')
        parser.add_argument('-v', '--verbose', '--info', dest='log_level',
                            action='store_const', const='info', default='warning',
                            help='enable informative (verbose) output, work on log level INFO')
        parser.add_argument('-d', '--debug', dest='log_level',
                            action='store_const', const='debug', default='warning',
                            help='enable debug output, work on log level DEBUG')
        levels = [i for i in logging._nameToLevel.keys()
                  if (type(i) == str and i != 'NOTSET')]
        parser.add_argument('--syslog', nargs='?', default=None,
                            type=str.upper, action='store',
                            const='INFO', choices=levels,
                            help="""log to syslog facility, default: no
                            logging, INFO if --syslog is specified without
                            argument""")
        parser.add_argument('--hashtaglist', dest='hashtaglist',
                            help='a list of hashtags to match')
        parser.add_argument('-p', '--populate-cache', action='store_true', default=False,
                            dest='populate',
                            help='populate RSS entries in cache without actually posting them to Mastodon')
        parser.add_argument('-r', '--rss', help='the RSS feed URL to fetch items from',
                            dest='rss_uri', metavar='http://...')
        parser.add_argument('--rss-sections', action='store_true', default=False,
                            dest='rsssections',
                            help='print the available sections of the rss feed to be used in the tweet template')
        self.opts = parser.parse_args()
        # expand the path to the cache file if defined
        if self.opts.cachefile:
            self.opts.cachefile = os.path.expanduser(self.opts.cachefile)
        # verify if the path to cache file is an absolute path
        # get the different config files, from a directory or from a *.ini style
        self.opts.config = list(map(os.path.expanduser, self.options.config))
        for element in self.opts.config:
            if element and not os.path.exists(element):
                sys.exit('You should provide an existing path for the config file: %s' % element)
            if os.path.isdir(element):
                self.opts.configs = glob.glob(os.path.join(element, '*.ini'))
            else:
                # trying to glob the path
                self.opts.configs = glob.glob(element)
        # verify if a configuration file is provided
        if not self.opts.configs:
            sys.exit('no configuration file was found at the specified path(s) with the option -c')
        # verify the path to the hashtaglist
        if self.opts.hashtaglist:
            hashtaglist = os.path.expanduser(self.opts.hashtaglist)
            if not os.path.exists(hashtaglist):
                sys.exit('the {hashtaglist} file does not seem to exist, please provide a valid path'.format(hashtaglist=hashtaglist))

    @property
    def options(self):
        '''return the path to the config file'''
        return self.opts



#!/usr/bin/env python3
# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Add as many tags as possible depending on the tweet length
'''Add as many tags as possible depending on the tweet length'''

# standard library imports
from operator import itemgetter

class AddTags:
    '''Add as many tags as possible depending on the tweet length'''
    def __init__(self, tweet, tags):
        '''Constructor of AddTags class'''
        self.tags = tags
        self.tweet = tweet
        self.main()

    def main(self):
        '''Main of the AddTags class class'''
        maxlength = 500
        tweetlength = len(self.tweet)

        # sort list of tags, the ones with the greater length first
        tagswithindices = ({'text':i, 'length': len(i)} for i in self.tags)
        sortedtagswithindices = sorted(tagswithindices, key=itemgetter('length'), reverse=True)
        self.tags = (i['text'] for i in sortedtagswithindices)

        # add tags is space is available
        for tag in self.tags:
            taglength = len(tag)
            if (tweetlength + (taglength + 1)) <= maxlength:
                self.tweet = ' '.join([self.tweet, tag])
                tweetlength += (taglength + 1)

    @property
    def finaltweet(self):
        '''return the final tweet with as many tags as possible'''
        return self.tweet



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Manage a lock file'''

# standard libraires imports
import datetime
import logging
import os
import os.path
import sys

class LockFile:
    '''LockFile object'''
    def __init__(self, lockfile, locktimeout):
        '''check the lockfile and the locktimeout'''
        self.lockfile = lockfile
        ltimeout = datetime.timedelta(seconds=locktimeout)
        self.lfdateformat = '%Y-%m-%d_%H-%M-%S'
        # if a lock file exists
        if os.path.exists(self.lockfile):
            if os.path.isfile(self.lockfile):
                with open(self.lockfile, 'r') as lf:
                    lfcontent = lf.read().rstrip()
                # lfcontent should be a datetime
                logging.debug('Check if lock file is older than timeout ({timeout} secs)'.format(timeout=locktimeout))
                locktime = datetime.datetime.strptime(lfcontent, self.lfdateformat)
                if locktime < (datetime.datetime.now() - ltimeout):
                    # remove the lock file
                    logging.debug('Found an expired lock file')
                    self.release()
                    self.create_lock()
                else:
                    # quit because another feed2toot process is running
                    logging.debug('Found a valid lock file. Exiting immediately.')
                    sys.exit(0)
        else:
            # no lock file. Creating one
            self.create_lock()

    def create_lock(self):
        '''Create a lock file'''
        with open(self.lockfile, 'w') as lf:
            currentdatestring = datetime.datetime.now().strftime(self.lfdateformat)
            lf.write(currentdatestring)
            logging.debug('lockfile {lockfile} created.'.format(lockfile=self.lockfile))

    def release(self):
        '''Release the lockfile'''
        os.remove(self.lockfile)
        logging.debug('Removed lock file.')



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Manage a cache with the ids of the feed entries'''

# standard libraires imports
from collections import deque
import os
import os.path

class FeedCache:
    '''FeedCache class'''

    def __init__(self, options):
        '''Constructore of the FeedCache class'''
        self.options = options
        self.main()

    def getdeque(self):
        '''return the deque'''
        return self.dbfeed

    def main(self):
        '''Main of the FeedCache class'''
        if os.path.exists(self.options['cachefile']):
            with open(self.options['cachefile']) as dbdsc:
                dbfromfile = dbdsc.readlines()
            dblist = [i.strip() for i in dbfromfile]
            self.dbfeed = deque(dblist, self.options['cache_limit'])
        else:
            self.dbfeed = deque([], self.options['cache_limit'])

    def append(self, rssid):
        '''Append a rss id to the cache'''
        self.dbfeed.append(rssid)

    def close(self):
        '''Close the cache'''
        with open(self.options['cachefile'], 'w') as dbdsc:
            dbdsc.writelines((''.join([i, os.linesep]) for i in self.dbfeed))



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the feedparser section
'''Get values of the feedparser section'''

# standard library imports
import sys

def parsefeedparser(config):
    '''Parse configuration values and get values of the feedparser section'''
    section = 'feedparser'
    option = 'accept_bozo_exceptions'
    accept_bozo_exceptions = False
    if config.has_option(section, option):
        accept_bozo_exceptions = config.getboolean(section, option)
    return accept_bozo_exceptions



# Copyright 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#!/usr/bin/env python3

# Setup for Feed2toot
'''Setup for Feed2toot'''

from setuptools import setup, find_packages

CLASSIFIERS = [
    'Intended Audience :: End Users/Desktop',
    'Environment :: Console',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
]

setup(
    name='feed2toot',
    version='0.11',
    license='GNU GPL v3',
    description='Parse rss feeds and send new posts to Mastodon',
    long_description='Parse rss feeds and send new posts to the Mastodon social network',
    author = 'Carl Chenet',
    author_email = 'chaica@ohmytux.com',
    url = 'https://gitlab.com/chaica/feed2toot',
    classifiers=CLASSIFIERS,
    download_url='https://gitlab.com/chaica/feed2toot',
    packages=find_packages(),
    scripts=['scripts/feed2toot', 'scripts/register_feed2toot_app'],
    install_requires=['beautifulsoup4', 'feedparser', 'Mastodon.py'],
    extras_require={
        'influxdb':  ["influxdb"]
    }
)



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get value of the patterne option of rss section
'''Get value of the addtags option of the rss section'''

# standard library imports
import logging
import sys

def parseaddtags(config):
    '''Parse configuration value of the addtags option of the rss section'''
    addtags = True
    section = 'rss'
    if config.has_section(section):
        if config.has_option(section, 'addtags'):
            try:
                addtags = config.getboolean(section, 'addtags')
            except ValueError as err:
                logging.warn(err)
                addtags = True
    return addtags



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the lock section
'''Get values of the lock section'''

# standard library imports
import os.path
import sys

def parselock(lockfile, locktimeout, config):
    '''Parse configuration values and get values of the hashtaglist section'''
    lockfile = lockfile
    locktimeout = locktimeout
    section = 'lock'
    ##################
    # lockfile option
    ##################
    confoption = 'lock_file'
    if config.has_section(section):
        lockfile = config.get(section, confoption)
    lockfile = os.path.expanduser(lockfile)
    lockfileparent = os.path.dirname(lockfile)
    if lockfileparent and not os.path.exists(lockfileparent):
        sys.exit('The parent directory of the lock file does not exist: {lockfileparent}'.format(lockfileparent=lockfileparent))
    ######################
    # lock_timeout option
    ######################
    if config.has_section(section):
        confoption = 'lock_timeout'
        if config.has_option(section, confoption):
            try:
                locktimeout = int(config.get(section, confoption))
            except ValueError as err:
                sys.exit('Error in configuration with the {confoption} parameter in [{section}]: {err}'.format(confoption=confoption, section=section, err=err))
    return lockfile, locktimeout



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get value of the uri_list option of rss section
'''Get value of the uri_list option of the rss section'''

# standard library imports
import feedparser
import logging
import os.path
import sys
import re

def parseurilist(config, accept_bozo_exceptions):
    '''Parse configuration value of the uri_list option of the rss section'''
    bozoexception = False
    feeds = []
    patterns = []
    section = 'rss'
    stringsep = ','
    if config.has_section(section):
        #################
        # uri_list option
        #################
        currentoption = 'uri_list'
        if config.has_option(section, currentoption):
            rssfile = config.get(section, currentoption)
            rssfile = os.path.expanduser(rssfile)
            if not os.path.exists(rssfile) or not os.path.isfile(rssfile):
                sys.exit('The path to the uri_list parameter is not valid: {rssfile}'.format(rssfile=rssfile))
            with open(rssfile, 'r') as rsfo:
                rsslist = rsfo.readlines()
            for line in rsslist:
                line = line.strip()
                # split each line in two parts, rss link and a string with the different patterns to look for
                feedname = ''
                if '<' in line:
                    matches = re.match('(.*) <(.*)>', line)
                    if not matches:
                        sys.exit('This line in the list of uri to parse is not formatted correctly: {line}'.format(line))
                    feedname, line = matches.groups()
                confobjects = line.split('|')
                if len(confobjects) > 3 or len(confobjects) == 2:
                    sys.exit('This line in the list of uri to parse is not formatted correctly: {line}'.format(line))
                if len(confobjects) == 3:
                    rss, rssobject, patternstring = line.split('|')
                if len(confobjects) == 1:
                    rss = confobjects[0]
                    rssobject = ''
                    patternstring = ''
                # split different searched patterns
                patterns = [i for i in patternstring.split(stringsep) if i]
                # retrieve the content of the rss
                feed = feedparser.parse(rss)
                if 'bozo_exception' in feed:
                    bozoexception = True
                    logging.warning(feed['bozo_exception'])
                    if not accept_bozo_exceptions:
                        continue
                # check if the rss feed and the rss entry are valid ones
                if 'entries' in feed:
                    if rssobject and rssobject not in feed['entries'][0].keys():
                        sys.exit('The rss object {rssobject} could not be found in the feed {rss}'.format(rssobject=rssobject, rss=rss))
                else:
                    sys.exit('The rss feed {rss} does not seem to be valid'.format(rss=rss))
                feeds.append({'feed': feed, 'patterns': patterns, 'rssobject': rssobject, 'feedname': feedname})
            # test if all feeds in the list were unsuccessfully retrieved and if so, leave
            if not feeds and bozoexception:
                sys.exit('No feed could be retrieved. Leaving.')
    return feeds



# -*- coding: utf-8 -*-
# Copyright © 2015-2019 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the cache section
'''Get values of the cache section'''

# standard library imports
import os.path
import sys

def parsecache(clioption, config):
    '''Parse configuration values and get values of the hashtaglist section'''
    cachefile = ''
    cachelimit = 100
    section = 'cache'
    if not clioption:
        ##################
        # cachefile option
        ##################
        confoption = 'cachefile'
        if config.has_section(section):
            cachefile = config.get(section, confoption)
        else:
            sys.exit('You should provide a {confoption} parameter in the [{section}] section'.format(section=section, confoption=confoption))
        cachefile = os.path.expanduser(cachefile)
        cachefileparent = os.path.dirname(cachefile)
        if cachefileparent and not os.path.exists(cachefileparent):
            sys.exit('The parent directory of the cache file does not exist: {cachefileparent}'.format(cachefileparent=cachefileparent))
    else:
        cachefile = clioption
    ####################
    # cache_limit option
    ####################
    if config.has_section(section):
        confoption = 'cache_limit'
        if config.has_option(section, confoption):
            try:
                cachelimit = int(config.get(section, confoption))
            except ValueError as err:
                sys.exit('Error in configuration with the {confoption} parameter in [{section}]: {err}'.format(confoption=confoption, section=section, err=err))
        else:
            cachelimit = 100
    return cachefile, cachelimit
